package com.swtjface.flowchart;

import java.io.*;
import java.util.EventObject;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.swt.SWT;
import org.eclipse.ui.*;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.gef.*;
import org.eclipse.gef.dnd.TemplateTransferDragSourceListener;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.parts.*;
import com.swtjface.flowchart.actions.FlowchartContextMenuProvider;
import com.swtjface.flowchart.dnd.FlowchartDropTargetListener;
import com.swtjface.flowchart.editpart.*;
import com.swtjface.flowchart.model.*;
import com.swtjface.flowchart.palette.*;
import org.eclipse.ui.IFileEditorInput;

public class FlowchartEditor extends GraphicalEditorWithPalette
{
  public FlowchartEditor()
  {
    DefaultEditDomain defaultEditDomain = new DefaultEditDomain(this);
    setEditDomain(defaultEditDomain);
  }

  Chart Flowchart;
  protected void setInput(IEditorInput input)
  {
    super.setInput(input);
    IFile file = ((IFileEditorInput)input).getFile();
    try
    {
      InputStream is = file.getContents(false);
      if(is.available()!=0)
      {
        ObjectInputStream ois = new ObjectInputStream(is);
        Flowchart = (Chart)ois.readObject();
        ois.close();
        setTitle(file.getName());
      } else
        Flowchart = new Chart();
    }

    catch (Exception e)
	{
      e.printStackTrace();
      Flowchart = new Chart();
    }
  }

  protected Chart getChart()
  {
    return Flowchart;
  }

  public void setChart(Chart chart)
  {
    Flowchart = chart;
  }

  protected void createOutputStream(OutputStream os)
    throws IOException
  {
    ObjectOutputStream outStream = new ObjectOutputStream(os);
    outStream.writeObject(getChart());
    outStream.close();
  }

  public void doSaveAs()
  {}

  public void doSave(IProgressMonitor monitor)
  {
    try
    {
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      createOutputStream(out);
      IFile file = ((IFileEditorInput)getEditorInput()).getFile();
      file.setContents(new ByteArrayInputStream(out.toByteArray()), true, false, monitor);
      out.close();
      getCommandStack().markSaveLocation();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  private boolean savePreviouslyNeeded = false;
  public boolean isDirty()
  {
    return isSaveOnCloseNeeded();
  }

  public boolean isSaveAsAllowed()
  {
    return true;
  }

  public boolean isSaveOnCloseNeeded()
  {
    return getCommandStack().isDirty();
  }

  private boolean savePreviouslyNeeded()
  {
    return savePreviouslyNeeded;
  }

  private void setSavePreviouslyNeeded(boolean value)
  {
    savePreviouslyNeeded = value;
  }

  protected void configureGraphicalViewer()
  {
    super.configureGraphicalViewer();
    getGraphicalViewer().setRootEditPart(new ScalableRootEditPart());
    getGraphicalViewer().setEditPartFactory(new PartFactory());
    ContextMenuProvider provider =
  	  new FlowchartContextMenuProvider(getGraphicalViewer(), getActionRegistry());
    getGraphicalViewer().setContextMenu(provider);
    getSite().registerContextMenu(provider, getGraphicalViewer());
    getGraphicalViewer().setKeyHandler(new GraphicalViewerKeyHandler(getGraphicalViewer())
      .setParent(getCommonKeyHandler()));
  }

  protected void initializeGraphicalViewer()
  {
    getGraphicalViewer().setContents(Flowchart);
    getGraphicalViewer().addDropTargetListener(
      new FlowchartDropTargetListener(getGraphicalViewer()));
  }

  private PaletteRoot root;
  protected PaletteRoot getPaletteRoot()
  {
    if (root == null)
      root = FlowchartPalette.getPaletteRoot();
    return root;
  }

  protected void initializePaletteViewer()
  {
    super.initializePaletteViewer();
    getPaletteViewer().addDragSourceListener(
      new TemplateTransferDragSourceListener(getPaletteViewer()));
  }

  // Part 4: Allow the user to interact with the environment
  private KeyHandler sharedKeyHandler;
  protected KeyHandler getCommonKeyHandler()
  {
    if (sharedKeyHandler == null)
    {
      sharedKeyHandler = new KeyHandler();
      sharedKeyHandler.put(KeyStroke.getPressed(SWT.DEL, 127, 0),
        getActionRegistry().getAction(ActionFactory.DELETE.getId()));
    }
    return sharedKeyHandler;
  }

  public void commandStackChanged(EventObject event)
  {
    if (isDirty())
    {
      if (!savePreviouslyNeeded())
      {
        setSavePreviouslyNeeded(true);
        firePropertyChange(IEditorPart.PROP_DIRTY);
      }
    }
    else
    {
      setSavePreviouslyNeeded(false);
      firePropertyChange(IEditorPart.PROP_DIRTY);
    }
    super.commandStackChanged(event);
  }

  public void gotoMarker(IMarker marker)
  { }
}
