package com.swtjface.flowchart.model;

import java.util.*;
import org.eclipse.draw2d.geometry.*;
import org.eclipse.ui.views.properties.*;

public class Activity
  extends AbstractChartElement implements IPropertySource
{

  // Information content of the Model class
  private String name;
  private Point location = new Point(0,0);
  private Dimension size = new Dimension(-1,-1);
  protected Vector targets = new Vector(2);
  protected Vector sources = new Vector(2);
  
  public Dimension getSize()
  {return size;}

  public void setSize(Dimension dim)
  {
    if (size.equals(dim)) return;
    size = dim;
    firePropertyChange(SIZE, null, size);
  }

  public Point getLocation() {return location;}
  public void setLocation(Point place)
  {
  	if (location.equals(place)) return;
  	location = place;
  	firePropertyChange(LOC, null, place);
  }
  
  public String getName()
  {return name;}
  
  public void setName(String str)
  {
  	if (str.equals(name)) return;
  	name = str;
  	firePropertyChange(NAME, null, str);
  }
  
  public Vector getConnections()
  {	
  	Vector v = getSourceConnections();
  	v.addAll(getTargetConnections());
  	return v;
  }
  
  public Vector getSourceConnections()
  {
    return (Vector)sources.clone();
  }

  public void addSourceConnection(Path p)
  {
    sources.add(p);
    firePropertyChange(SOURCES, null, p);
  }

  public void removeSourceConnection(Path p)
  {
    sources.remove(p);
    firePropertyChange(SOURCES, null, p);
  }

  public Vector getTargetConnections()
  {
    return (Vector)targets.clone();
  }
  
  public void addTargetConnection(Path p)
  {
    targets.add(p);
    firePropertyChange(TARGETS, null, p);
  }
  
  public void removeTargetConnection(Path p)
  {
    targets.remove(p);
    firePropertyChange(TARGETS, null, p);
  }
  
  public Object getEditableValue()
  {return null;}
  
  public IPropertyDescriptor[] getPropertyDescriptors()
  {
    return new IPropertyDescriptor[]
    {
      new TextPropertyDescriptor(NAME, "Name"),
      new TextPropertyDescriptor(SIZE, "Size"),
      new TextPropertyDescriptor(LOC, "Location")
    };
  }
  
  public Object getPropertyValue(Object propName)
  {
    if (propName.equals(SIZE))
      return getSize().width + "," + getSize().height;
    else if(propName.equals(LOC))
      return getLocation().x + "," + getLocation().y;
    else if(propName.equals(NAME))
      return getName();
    return null;
  }
  
  public void setPropertyValue(Object propName, Object value)
  {
    String str = (String)value;
    int comma = str.indexOf(",");
    if (propName.equals(SIZE))
      setSize(new Dimension(Integer.parseInt
        (str.substring(0,comma)), Integer.parseInt(str.substring(comma+1))));
    else if (propName.equals(LOC))
      setLocation(new Point(Integer.parseInt
        (str.substring(0,comma)),
      Integer.parseInt(str.substring(comma+1))));
    else if (propName.equals(NAME))
      setName(str);
  }
  
  public boolean isPropertySet(Object id)
  {return true;}
  
  public void resetPropertyValue(Object id)
  {}
}